#ifndef __DESCRIPTOR_TABLE_INCLUDED__
#define __DESCRIPTOR_TABLE_INCLUDED__

/*******************************
 * ia32 descriptor table stuff *
 *******************************/
#define	D_LDT		0x0200
#define D_TASK		0x0500
#define	D_TSS		0x0900
#define	D_CALL		0x0C00
#define	D_INT		0x0E00
#define	D_TRAP		0x0F00
#define	D_DATA		0x1000
#define	D_CODE		0x1800
#define	D_DPL3		0x6000
#define	D_DPL2		0x4000
#define	D_DPL1		0x2000
#define	D_PRESENT	0x8000
#define	D_NOT_PRESENT	0x8000
#define	D_ACC		0x0100
#define	D_WRITE		0x0200
#define	D_READ		0x0200
#define	D_BUSY		0x0200
#define	D_EXDOWN	0x0400
#define	D_CONFORM	0x0400
#define	D_BIG		0x0040
#define	D_BIG_LIM	0x0080

struct ia32_desc {
	u_int16_t	limit_low;
	u_int16_t	base_low;
	u_int8_t	base_med;
	u_int8_t	access;
	u_int32_t	limit_high:4;
	u_int32_t	granularity:4;
	u_int8_t	base_high;
} __attribute__ ((packed));

struct ia32_gate {
	u_int16_t	offset_low;
	u_int16_t	selector;
	u_int16_t	access;
	u_int16_t	offset_high;
} __attribute__ ((packed));

typedef union dt_entry {
	struct ia32_desc desc;
	struct ia32_gate gate;
	u_int64_t dummy;
}dt_entry_t;

/* Macro to easily create descriptors */
#define stnd_desc(base, limit, control) \
	{desc: {(limit & 0xffff), (base & 0xffff), \
	((base >> 16) & 0xff), \
	((control+D_PRESENT) >> 8), (limit >> 16),\
	((control & 0xff) >> 4), (base >> 24)}}
	
#define gate_desc(offset, selector, control) {gate: {(offset & 0xffff), \
	selector, (control+D_PRESENT), (offset >> 16) }}

#endif /* __DESCRIPTOR_TABLE_INCLUDED__ */
