#include <kernel.h>
#include <arch/8259a.h>
#include <arch/idt.h>

pic_t pics[]={
	{0x20, 0x21, 0xFF, VECTOR_M, NULL, {1<<2}},	/* Master PIC in PC/AT */
	{0xA0, 0xA1, 0xFF, VECTOR_S, NULL, {2}},	/* Slave PIC in PC/AT */
};

void __init pic_init(void)
{
	/* Setup cascade relationships */
	pics[1].master=&pics[0];
	
}

u_int32_t __init pic_probe(void)
{
	/* No real probing to be done */
	return (sizeof(pics)/sizeof(pic_t));
}

u_int32_t __init pic_setup(void)
{
	int i;

	cli();

	/* Initialise PICs */
	for ( i=0; i<(sizeof(pics)/sizeof(pic_t)); i++) {
		outb(pics[i].port, ICW1);
		outb(pics[i].port_imr, pics[i].vector);

		if ( pics[i].master ) {
			outb(pics[i].port_imr, pics[i].cascade.master_pin);
		}else{
			outb(pics[i].port_imr, pics[i].cascade.slave_pins);
		}
		
		outb(pics[i].port_imr, ICW4);
		outb(pics[i].port_imr, pics[i].imr);
	}

	idt_init();
	sti();
	return i;
}

void pic_unmask(p_pic pic, u_int8_t bit)
{
	MASK_UNSET(pic->imr, bit);
	if ( pic->master ) pic_unmask(pic->master, pic->cascade.master_pin);
}

void pic_mask(p_pic pic, u_int8_t bit)
{
	MASK_SET(pic->imr, bit);
	if ( pic->master ) pic_mask(pic->master, pic->cascade.master_pin);
}

void pic_sync_imr(p_pic pic)
{
	outb(pic->port_imr, pic->imr);
	if ( pic->master ) pic_sync_imr(pic->master);
}

void pic_eoi_cascade(p_pic pic, u_int32_t pin)
{
	if ( pic->master ) pic_eoi_cascade(pic->master, pic->cascade.master_pin);
	outb(pic->port, EOI_SPECIFIC+pin);
}

void pic_eoi(p_pic pic)
{
	if ( pic->master ) pic_eoi_cascade(pic->master, pic->cascade.master_pin);
	outb(pic->port, EOI);
}

/* ==========================
*   PC/AT Specific commands 
*  ========================== */
void irq_on(u_int16_t irq)
{
	if ( irq < 8 ) {
		pic_unmask(&pics[0], irq);
		pic_sync_imr(&pics[0]);
	}else{
		pic_unmask(&pics[1], irq-8);
		pic_sync_imr(&pics[1]);
	}
}

void irq_off(u_int16_t irq)
{
	if ( irq < 8 ) {
		pic_mask(&pics[0], irq);
		pic_sync_imr(&pics[0]);
	}else{
		pic_mask(&pics[1], irq-8);
		pic_sync_imr(&pics[1]);
	}
	
}

inline void irq_eoi(u_int16_t irq)
{
	if ( irq < 8 ) {
		pic_eoi(&pics[0]);
	}else{
		pic_eoi(&pics[1]);
	}
}
