#include <kernel.h>
#include <arch/idt.h>
#include <arch/8259a.h>
#include <arch/setup.h>
#include <arch/descriptor.h>

/* IDT structure */
dt_entry_t IDT[256];
struct {
	unsigned short limit __attribute__ ((packed));
	union dt_entry *idt __attribute__ ((packed));
} loadidt={ (256*sizeof(dt_entry_t) - 1), IDT};

/* NULL interrupt vector */
asm(	".globl int_null\n"
	"int_null:\n"
	"	iret\n");

/* Add a new trap vector to the IDT  */
void idt_trap(void *handler, u_int8_t intr)
{
	IDT[intr].gate.selector		= 0x08;
	IDT[intr].gate.offset_low	= (u_int16_t)(((u_int32_t)handler)&0xffff);
	IDT[intr].gate.offset_high 	= (u_int16_t)(((u_int32_t)handler)>>16);
	IDT[intr].gate.access		= D_PRESENT | D_TRAP | D_DPL3;
}

/* Add a new interrupt vector to the IDT  */
void idt_interrupt(void *handler, u_int8_t intr)
{
	IDT[intr].gate.selector		= 0x08;
	IDT[intr].gate.offset_low	= (u_int16_t)(((u_int32_t)handler)&0xffff);
	IDT[intr].gate.offset_high 	= (u_int16_t)(((u_int32_t)handler)>>16);
	IDT[intr].gate.access		= D_PRESENT | D_INT;
}

void __init idt_init(void)
{
	int count;

	/* Stick all our vectors in to the IDT */
	idt_trap(_exc00, 0x00);
	idt_trap(_exc01, 0x01);
	idt_trap(_exc02, 0x02);
	idt_trap(_exc03, 0x03);
	idt_trap(_exc04, 0x04);
	idt_trap(_exc05, 0x05);
	idt_trap(_exc06, 0x06);
	idt_trap(_exc07, 0x07);
	idt_trap(_exc08, 0x08);
	idt_trap(_exc09, 0x09);
	idt_trap(_exc0a, 0x0a);
	idt_trap(_exc0b, 0x0b);
	idt_trap(_exc0c, 0x0c);
	idt_trap(_exc0d, 0x0d);
	idt_trap(_exc0e, 0x0e);
	idt_trap(_exc0f, 0x0f);
	idt_trap(_exc10, 0x10);
	idt_trap(_exc11, 0x11);
	idt_trap(_exc12, 0x12);
	idt_trap(_exc13, 0x13);
	
	/* Nullify the other vectors */ 
	for(count=0x14; count<256; count++) {
		idt_trap(int_null, count);
	}

	/* Load the IDT and reenable interrupts for
	* the first time. This is where things could explode  */
	asm("lidt (loadidt)\n");
}

void _exc00()
{
	kprint("(int:0x00): Divide Error\n");
	enter_to_exit();
}

void _exc01()
{
	kprint("(int:0x01): Debug exception\n");
	enter_to_exit();
}

void _exc02()
{
	kprint("(int:0x02): Non-Maskable Interrupt\n");
	enter_to_exit();
}

void _exc03()
{
	kprint("(int:0x03): Breakpoint\n");
	enter_to_exit();
}

void _exc04()
{
	kprint("(int:0x04): Overflow\n");
	enter_to_exit();
}

void _exc05()
{
	kprint("(int:0x05): Bounds check\n");
	enter_to_exit();
}

void _exc06()
{
	kprint("(int:0x06): Invalid opcode\n");
	enter_to_exit();
}

void _exc07()
{
	kprint("(int:0x07): Coprocessor not avaulable\n");
	enter_to_exit();
}

void _exc08()
{
	kprint("(int:0x08): Double fault\n");
	enter_to_exit();
}

void _exc09()
{
	kprint("(int:0x09): Coprocessor segment overrun\n");
	enter_to_exit();
}

void _exc0a()
{
	kprint("(int:0x0a): Invalid TSS\n");
	enter_to_exit();
}

void _exc0b()
{
	kprint("(int:0x0b) Segment not present\n");
	enter_to_exit();
}

void _exc0c()
{
	kprint("(int0x0c): Stack exception\n");
	enter_to_exit();
}

void _exc0d()
{
	kprint("(int:0x0d): General Protection Fault (w00p!)\n");
	enter_to_exit();
}

void _exc0e()
{
	kprint("(int:0x0e): Page Fault\n");
	enter_to_exit();
}

void _exc0f()
{
	kprint("(int:0x0f): Unknown error\n");
	enter_to_exit();
}

void _exc10()
{
	kprint("(int:0x10): Coprocessor error\n");
	enter_to_exit();
}

void _exc11()
{
	kprint("(int:0x11): Alignment check\n");
	enter_to_exit();
}

void _exc12()
{
	kprint("(int:0x12): Machine check\n");
	enter_to_exit();
}

void _exc13()
{
	kprint("(int:0x13): SSE fault\n");
	enter_to_exit();
}
