/* Copyright (c) 2001 Greg Sheard <greg@ecsc.co.uk>*/
/* pci_base.c: initialise the PCI buses and export symbols */

#include <kernel.h>
#include <arch/pci.h>

#define  _PCI_PREFIX "  (bus/pci): "

#define PCICMD(bus, dev, fn)	(0x80000000 | (bus << 16) | (dev << 11) | (fn << 3))

/* PCI vendor id list */
pci_vendor_t pcivend[]={
	{"ATI Technologies Inc", 0x1002, {
		{"3D Rage IIC AGP", 0x4757, PCI_FLAGS_DEVICE_AGP | PCI_FLAGS_TYPE_VGA},
					 }
	},
	{"Digital Equipment Corporation", 0x1011, {
		{"DECchip 21040 (Tulip)", 0x0002, PCI_FLAGS_DEVICE_PCI | PCI_FLAGS_TYPE_ETHER},
		{"DECchip 21152", 0x0024, PCI_FLAGS_DEVICE_PCI | PCI_FLAGS_TYPE_ETHER},
						  },
	},
	{"VIA Technologies, Inc", 0x1106, {
		{"VT82C585VP Apollo VP1/VPX", 0x585, PCI_FLAGS_BRIDGE_HOST},
		{"VT82C8586A/B Apollo VP", 0x586, PCI_FLAGS_BRIDGE_ISA},
					  },
	},
	{"Lucent Microelectronics", 0x11C1, {
		{"Venus Modem", 0x0480, PCI_FLAGS_DEVICE_PCI | PCI_FLAGS_TYPE_COMMS},
					    },
	},
	{"ASIX Electronics", 0x125B, {
		{"AX88140", 0x1400, PCI_FLAGS_DEVICE_PCI | PCI_FLAGS_TYPE_ETHER},
				     },
	},
	{"S3 Inc", 0x5333, {
		{"86c764/765 Trio32/64/64V+", 0x8811, PCI_FLAGS_DEVICE_PCI | PCI_FLAGS_TYPE_VGA},
			   },
	}, 
	{"Intel Corporation", 0x8086, {
		{"440BX/ZX - 82443BX/ZX", 0x7190, PCI_FLAGS_BRIDGE_HOST},
		{"440BX/ZX - 82443BX/ZX", 0x7191, PCI_FLAGS_BRIDGE_AGP},
		{"82371AB PIIX4", 0x7110, PCI_FLAGS_BRIDGE_ISA},
		{"82371AB PIIX4 IDE", 0x7111, PCI_FLAGS_TYPE_IDE},
		{"82371AB PIIX4 USB", 0x7112, PCI_FLAGS_TYPE_USB},
		{"82371AB PIIX4", 0x7113, PCI_FLAGS_BRIDGE},
		{"82557 [Ethernet Pro 100]", 0x1229, PCI_FLAGS_DEVICE_PCI | PCI_FLAGS_TYPE_ETHER},
		{"430VX - 82437VX TVX", 0x7030, PCI_FLAGS_BRIDGE_HOST},
		{"82371SB PIIX3", 0x7000, PCI_FLAGS_BRIDGE_ISA},
		{"82371SB PIIX3 IDE", 0x7010, PCI_FLAGS_TYPE_IDE},
		{"82371SB PIIX3 USB", 0x7020, PCI_FLAGS_TYPE_USB},
		{"82810-DC100 GMCH", 0x7122, PCI_FLAGS_BRIDGE_HOST},
		{"82810-DC100 CGC", 0x7123, PCI_FLAGS_BRIDGE_AGP},
		{"82801AA", 0x2410, PCI_FLAGS_BRIDGE_ISA},
		{"82801AA", 0x2418, PCI_FLAGS_BRIDGE_PCI},
				      },
	},
};

u_int32_t conf;

void pci_init(void)
{
}

u_int32_t pci_probe(void)
{
	cli();
	
	if ( (conf=pci_probe_direct()) ) {
		sti();
		return 1;
	}

	sti();
	return 0;
}

u_int32_t maxbus=0, busfound=0, buschecked=0;

u_int32_t pci_setup(void)
{
	u_int32_t tmp;
	u_int32_t ret;
	u_int32_t rval;
	u_int32_t devcount=0;
	u_int32_t bus=0, dev=0, func=0;
	u_int32_t dev_found_in_bus=0;

	cli();

	/* so we can tidy up before we go */
	tmp = inl(0x0cf8);

	ret = pci_setup_getdev(PCICMD(bus, dev, func));
	rval = pci_setup_identify(ret);
	if (!rval)
	{
		kprint(_PCI_PREFIX "Error, no host bridge found.\n");
		sti();
		return 0;
	}else{
		devcount+=rval;
	}
	
	for (bus=0x00; bus<=(maxbus-1); bus++)
	{
		for (dev=0x01; dev<=0x1F; dev++)
		{
			for (func=0x0; func<=0x07; func++)
			{
				ret = pci_setup_getdev(PCICMD(bus, dev, func));
				rval = pci_setup_identify(ret);
				devcount+=rval;
				dev_found_in_bus+=rval;
			}
		}
		if (dev_found_in_bus) {
			buschecked++;
		}else{
			maxbus++;
		}
		dev_found_in_bus = 0;
		if (bus > 0x0F) {
			bus = maxbus;
		}
	}
	
	/* clean up - put it back how we found it */
	outl(0x0cf8, tmp);

	sti();

	kprint(_PCI_PREFIX "Buses found: %u.\n", buschecked);
	kprint(_PCI_PREFIX "Total devices found: %u.\n", devcount);
	return devcount;
}

u_int32_t pci_setup_identify(u_int32_t pci_ret)
{
	u_int16_t vendor, device;
	u_int32_t i=0, j=0;

        vendor = (u_int16_t) pci_ret&0xFFFF;
        device = pci_ret>>16;

	/* Special case handlers: if the values are 0xFFFF, there's no device */
	/* Some PCI bridges seem to report 0x0 instead of 0xFFFF... */
	if ( (vendor == PCI_VENDOR_NODEVICE || device == PCI_DEVCODE_NODEVICE) ||
			(vendor == PCI_VENDOR_ANY || device == PCI_DEVCODE_ANY) )
	{
		return 0;
	}
	
	for (i=0; i<(sizeof(pcivend)/sizeof(pci_vendor_t)); i++)
	{
		if (vendor == pcivend[i].vendor)
		{
			for (j=0; j<(sizeof(pcivend[i].devlist)/sizeof(pci_device_t)); j++)
			{
				if (pcivend[i].devlist[j].device != 0) {
					if (device == pcivend[i].devlist[j].device)
					{
						kprint(_PCI_PREFIX "%s: %s %s\n",
							pf(pcivend[i].devlist[j].flags),
							pcivend[i].name,
							pcivend[i].devlist[j].name);
						return 1;
					}
				}
			}
			kprint(_PCI_PREFIX "Unknown: %s, device 0x%x\n",
					pcivend[i].name,
					device);
		}
	}
	
	return 0;
}

u_int32_t pci_setup_getdev(u_int32_t devnum)
{
	u_int32_t tmp, ret;
        tmp = inl(0xcf8);
        outl(0xcf8, devnum); 
        ret = inl(0x0cfc);
	return ret;
}
u_int32_t pci_probe_direct(void)
{
	u_int32_t tmp;

	/* Configuration 1 */
	outb(0x0cfB, 0x01);
	tmp = inl(0x0cf8);
	outl(0x0cf8, 0x80000000);
	if (inl(0x0cf8) == 0x80000000) {
		outl(0x0cf8, tmp);
		kprint(_PCI_PREFIX "Using configuration 1\n");
		return 1;
	}
	outl(0x0cf8, tmp);

	/* Configuration 2 */
	outb (0xCFB, 0x00);
	outb (0x0cf8, 0x00);
	outb (0x0cfa, 0x00);
	if ( (inb(0x0cf8) == 0x00) && 
		(inb(0xCFA) == 0x00)) {
		kprint(_PCI_PREFIX "Using configuration 2 (deprecated)\n");
		return 2;
	}

	return 0;
}

char *pf(u_int32_t pciFlags)
/* TODO: Make this able to test multiple flags! */
{
	if (pciFlags & PCI_FLAGS_BRIDGE_HOST) {
		maxbus++; busfound++;
		return ("Host bridge");
	};
	if (pciFlags & PCI_FLAGS_BRIDGE_ISA) {
		maxbus++; busfound++;
		return ("ISA bridge");
	};
	if (pciFlags & PCI_FLAGS_BRIDGE_AGP) {
		maxbus++; busfound++;
		return ("AGP bridge");
	};
	if (pciFlags & PCI_FLAGS_BRIDGE_PCI) {
		maxbus++; busfound++;
		return ("PCI bridge");
	};
	if (pciFlags & PCI_FLAGS_DEVICE_PCI) {
		return ("PCI device");
	};
	if (pciFlags & PCI_FLAGS_DEVICE_ISA) {
		return ("ISA device");
	};
	if (pciFlags & PCI_FLAGS_DEVICE_AGP) {
		return ("AGP device");
	};
	if (pciFlags & PCI_FLAGS_DEVICE_OTHER) {
		return ("Other device");
	};
	return ("Unknown type");
}

u_int32_t pci_init_bus(u_int32_t idBus)
{
	return idBus;
}

u_int32_t pci_halt_bus(u_int32_t idBus)
{
	return idBus;
}

