#ifndef __VFS_HEADER_INCLUDED__
#define __VFS_HEADER_INCLUDED__

#include <types.h>

#ifdef CONFIG_VFS_64_BIT
typedef u_int64_t ofs_t;
#else
typedef u_int32_t ofs_t;
#endif

#define SEEK_SET	0
#define SEEK_CUR	1

typedef enum itype{
	ITYPE_DIR,
	ITYPE_FILE,
	ITYPE_BLOCK,
	ITYPE_CHAR,
	ITYPE_SYMLINK,
	ITYPE_META,
	ITYPE_MAX
}itype_t;

typedef struct inode {
	itype_t		type;
	u_int32_t	major;
	u_int32_t	minor;
}inode_t, *p_inode;

typedef struct super {
}super_t, *p_super;

typedef struct statfs {
}statfs_t, *p_statfs;

/* Superblock ops */
typedef void (*proc_read_inode)		(p_inode);
typedef void (*proc_put_inode)		(p_inode);
typedef void (*proc_delete_inode)	(p_inode);
typedef struct super_ops {
	proc_read_inode			read_inode;
	proc_put_inode			put_inode;
	proc_delete_inode		delete_inode;
}super_ops_t, *p_super_ops;

/* Inode ops */

/* Internal representation of a file */
typedef struct file {
	struct file_ops	*f_op;
	ofs_t		f_pos;
	void		*priv;
}file_t, *p_file;

/* FIle ops */
typedef struct file_ops{
	int32_t (*open)	(p_inode, p_file);
	int32_t (*close)(p_file);
	int32_t (*read)	(p_file, char *, ofs_t);
	int32_t (*write)(p_file, char *, ofs_t);
	int32_t (*lseek)(p_file, ofs_t, u_int32_t);
}file_ops_t, *file_ops;

/* Block device */
#define RWB_READ	0
#define RWB_WRITE	1
typedef int32_t (*proc_blk_rw)(int32_t, u_int32_t, char *, u_int32_t);
typedef struct blkdev {
	u_int32_t	major;
	u_int32_t	minor;
	proc_blk_rw	ll_rw_block;
	u_int32_t	blk_size;
}blkdev_t, *p_blkdev;

/* Block device functions */
extern file_ops_t blk_ops;


/* VFS functions */
int32_t vfs_open(p_inode, p_file);

#endif /* __VFS_HEADER_INCLUDED__ */
