#ifndef __KERNEL_HEADER_INCLUDED__
#define __KERNEL_HEADER_INCLUDED__

#include <config.h>
#include <types.h>
#include <errno.h>
#include <driver.h>
#include <lib/kprint.h>
#include <lib/ksym.h>
#include <lib/malloc.h>
#include <lib/list.h>

#define __init		__attribute__ ((__section__ (".text.init")))
#define __initdata	__attribute__ ((__section__ (".data.init")))


#define MASK(flags, bit) 		((flags) & (1<<(bit)))
#define MASK_SET(flags, bit)		((flags) |= (1<<(bit)))
#define MASK_UNSET(flags, bit)		((flags) &= ~(1<<(bit)))

extern u_int32_t ticks;

#ifdef ARCH_IA32
typedef enum monitor_status {
	MONITOR_NONE,
	MONITOR_COLOUR,
	MONITOR_MONO,
	MONITOR_UNKNOWN
}monitor_status_t;
extern char *monitor_str[];
#endif 

typedef enum boot_method {
	BM_BOOTLOADER,
	BM_MULTIBOOT
}boot_method_t;
extern char *boot_str[];

typedef struct kboot {
	char			*cmd_line;
	unsigned long		mem_lo;
	unsigned long		mem_up;
	unsigned long		mem_size;
	boot_method_t		boot_method;
	u_int32_t		smp;

	/* Kernel addresses */
	u_int32_t		kernel_addr;
	u_int32_t		kernel_end;
	u_int32_t		ksym_start;
	u_int32_t		ksym_end;

	
#ifdef ARCH_IA32
	monitor_status_t	monitor;
#endif

}kboot_t;

extern kboot_t boot_info;
void cli(void);
void sti(void);

#endif /* __KERNEL_HEADER_INCLUDED__ */
